# Instumental Variable Models for Discrete Outcomes, Andrew Chesher, November 2nd 2009.
#
# this version: 2 Nov 2009
#
# drawbounds1 draws the identified set for a single value of an instrumental variable
# a1 is Pr[Y=0|X=0,Z=z]
# a2 is Pr[Y=0|X=1,Z=z]
# d1 is Pr[X=0|Z=z]
# ----------------------------------------------------------------------------

drawbounds1 = function(a1,a2,d1,newpic =T, fill=T) {
    if (newpic) {plot(c(0,1),c(0,1),type='n',xaxs='i',yaxs='i',xlab = "p(0)",ylab = "p(1)",cex=2)}
    d2 = 1-d1
    if (fill) {rect(xleft = d1*a1,xright = d1*a1+d2*a2,ybottom = d1*a1+d2*a2, ytop =d1+d2*a2,col="red")
        rect(xleft = d1*a1+d2*a2,xright = d1*a1+d2,ybottom = d2*a2, ytop =d1*a1+d2*a2,col="blue")} else {
        rect(xleft = d1*a1,xright = d1*a1+d2*a2,ybottom = d1*a1+d2*a2, ytop =d1+d2*a2,col=NA)
        rect(xleft = d1*a1+d2*a2,xright = d1*a1+d2,ybottom = d2*a2, ytop =d1*a1+d2*a2,col=NA)}
    abline(0,1,col="red")
    }

# Example call:
# drawbounds1(.3,.7,.6)

# getprobs calculates probabilities a1, a2, d1, as above in a Gaussian triangular model for binary Y and binary X.
# The model has Y = 1[a0 + a1*x + u>0], X=1[b0 + b1 *z + v >0] and (U,V) Gaussian, mean zero variances one and correlation rho.

# Uses the pmnorm bivariate normal probability function from the R package mvnormt. That must be installed and loaded.
# Here is the load command.

library(mnormt)

getprobs = function(z,a0=0,a1=1,b0=0,b1=1,rho=0) {
    d1 = pnorm(-b0-b1*z)
    d2 = 1-d1
    a1d1 = pmnorm(c(-a0,-b0-b1*z),mea = c(0,0), varcov = rbind(c(1,rho),c(rho,1)))
    a2d2 = pnorm(-a0-a1) - pmnorm(c(-a0-a1,-b0-b1*z),mea = c(0,0), varcov = rbind(c(1,rho),c(rho,1)))
    a1=a1d1/d1
    a2=a2d2/d2
    return(list(a1=a1,a2=a2,d1=d1,z=z))
    }

# Example call
# getprobs(0)
# returns $a1 = .5 $a2 =  0.1586553 $d1 = .5, $rho = 0


# arect draws a rectangle if there is a rectangle to be drawn
# the function is used to draw intersections of identified sets if they are nonempty.

arect = function(xl,xr,yt,yb,colour=NA) {
    if ((xl <xr) & (yb < yt)) {rect(xleft = xl,xright = xr, ytop = yt, ybottom = yb, col=colour)}
    }

# drawbounds5 draws a sequence of identified sets associated with a sequence of instrumental values specified in zseq.
# parameter values for the structure generating the probability distribution employed in the calculations are a0, a1, b0, b1 and rho specified as in the function getprobs.
# if intersect=T then the intersection of the various sets is drawn once the end of the sequence is reached.

drawbounds5 = function(zseq,a0=0,a1=1,b0=0,b1=1,rho=0,intersect=T,fill=T,writetext=NA) {
    
    count = 0
    newpic = T
    vd1=NULL;va1=NULL;va2=NULL;vd2=NULL
    for (z in zseq) {
        count = count + 1
        if (count > 1) { newpic = F}
        aa = getprobs(z,a0=a0,a1=a1,b0=b0,b1=b1,rho=rho)
        vd1 = c(vd1,aa$d1)
        va1 = c(va1,aa$a1)
        va2 = c(va2,aa$a2)
        drawbounds1(aa$a1,aa$a2,aa$d1,newpic = newpic,fill=fill)
        abline(v=pnorm(-a0),col="green",lwd=2)
        abline(h=pnorm(-a0-a1),col="green",lwd=2)
        points(pnorm(-a0),pnorm(-a0-a1),pch=19,cex=1.5)
        
        }
    if (intersect) { vd2 = 1 - vd1
        arect(xl = max(vd1*va1),xr = min(vd1*va1+vd2*va2),yb = max(vd1*va1+vd2*va2), yt = min(vd1+vd2*va2) ,col="yellow")
        arect(xl = max(vd1*va1+vd2*va2) ,xr = min(vd1*va1+vd2) ,yb = max(vd2*va2) , yt = min(vd1*va1+vd2*va2) ,col="yellow")
        abline(v=pnorm(-a0),col="green",lwd=2)
        abline(h=pnorm(-a0-a1),col="green",lwd=2)
        points(pnorm(-a0),pnorm(-a0-a1),pch=19,cex=1.5) } 
    if (!is.na(writetext)) {text(0.1,0.9,writetext,cex=1.5,pos=4)}
    }


