# Instumental Variable Models for Discrete Outcomes, Andrew Chesher, November 2nd 2009.
#
# this version: 2 Nov 2009
#
# ----------------------------------------------------------------------------
# function to be integrated. It depends on parameters of the data generating process.
# there has been a transformation from a1*x to w - this way we don't have to make exceptions for positive and negative a1
# However the case with a1 zero will have to be treated separately.
# ----------------------------------------------------------------------------
integrand = function(w,zvalue,cc,a,b,suv,svv,atrue) {
	argpn <- cc - a[1] - a[2]*w/atrue[2] - (suv/svv)*(w/atrue[2] - b[1] - b[2]*zvalue)
	pn <- pnorm(argpn/sqrt(1-suv*suv/svv))*(1/sqrt(svv))*(1/abs(atrue[2]))

	argdd <- (w/atrue[2] - b[1] - b[2]*zvalue)/sqrt(svv)
	dd <- dnorm(argdd) 

	res <- pn*dd
return(res)
}

# lower and upper bounding functions
# tau is the probability level. z is the value of the instrumental variable.

sluz <- function(z,tau,Tm,a,b,suv,svv,atrue,subdivisions = 100){
	qtau = qnorm(tau)
	cvalues = c(-Inf, Tm, Inf)
	nintervals = length(cvalues) - 1

	if(atrue[2] != 0){
		answerl = 0
		answeru = 0
		for (i in 1:nintervals){
			low = cvalues[i] - atrue[1] - qtau
			upp = cvalues[i+1] - atrue[1] - qtau

			intresultl = integrate(integrand,lower=low,upper=upp,zvalue = z,
				cc = cvalues[i],a=a,b=b,suv=suv,svv=svv,atrue=atrue,subdivisions = subdivisions)
			intresultu = integrate(integrand,lower=low,upper=upp,zvalue = z,
				cc = cvalues[i+1],a=a,b=b,suv=suv,svv=svv,atrue=atrue,subdivisions = subdivisions)

			contributionl = intresultl$value
			answerl = answerl + contributionl
			contributionu = intresultu$value
			answeru = answeru + contributionu
		}
	}

	if (atrue[2] == 0) {
		answerl = 0
		answeru = 0
		for (i in 1:nintervals) {
			contributionl = 0
			contributionu = 0
			if (((cvalues[i] - atrue[1]) < qtau) && (qtau <= (cvalues[i+1] - atrue[1]))) {
				contributionl = pnorm((cvalues[i] - (a[1] + a[2]*b[1] + a[2]*b[2]*z))/sqrt(1+2*a[2]*suv + (a[2]^2)*svv))
				contributionu = pnorm((cvalues[i+1] - (a[1] + a[2]*b[1] + a[2]*b[2]*z))/sqrt(1+2*a[2]*suv + (a[2]^2)*svv))
			}
			answerl = answerl + contributionl
			answeru = answeru + contributionu
		}
	}
return (list(l = answerl,u = answeru))
}       
# ----------------------------------------------------------------------------          
# lower and upper bounding functions optimised over z in some interval
bzuppfun = function (z,tau,Tm,a,b,suv,svv,atrue,subdivisions = 100){
     answer  <- sluz(z,tau,Tm,a,b,suv,svv,atrue,subdivisions = 100)$u
     return(answer)
}

bzlowfun = function (z,tau,Tm,a,b,suv,svv,atrue,subdivisions = 100){
     answer = sluz(z,tau,Tm,a,b,suv,svv,atrue,subdivisions = 100)$l
     return(answer)
}

boundsz = function(zmin,zmax,tau,Tm,a,b,suv,svv,atrue,subdivisions = 100){
    ubanswer = optimize(bzuppfun,lower = zmin,upper = zmax,maximum = F,Tm=Tm,tau=tau,a=a,b=b,suv=suv,svv=svv,atrue=atrue)
    lbanswer = optimize(bzlowfun,lower = zmin,upper = zmax,maximum = T,Tm=Tm,tau=tau,a=a,b=b,suv=suv,svv=svv,atrue=atrue)
    
    return(list(ub = ubanswer,lb = lbanswer))
}
# ----------------------------------------------------------------------------          
# evaluates over a grid of alpha0 and alpha1 to determine which points on the grid are in the identified set.
# this can be very slow -e.g. 6 hours for a 100*100 grid.

gridit = function(atruelist,zmin,zmax,tauvec,Tm,a,b,suv,svv){
	a0vec <- atruelist[[1]]
	a1vec <- atruelist[[2]]
	aa <<- a0vec
     n0 <- length(a0vec)
     n1 <- length(a1vec)
     answer <- matrix(1,nrow=n0,ncol=n1)
	a.l <<- answer
	for (i in 1:length(a0vec)){
		for (j in 1:length(a1vec)){
			for (tau in tauvec) {
				print(c(i,j))
				aa <- boundsz(zmin,zmax,tau,Tm,a,b,suv,svv,c(a0vec[i],a1vec[j]) )
				if (aa$ub$objective <tau || aa$lb$objective >tau) { answer[i,j] <- 0; break }
			}
		}
	}
return(answer)
}
# ----------------------------------------------------------------------------          
plotgrid = function(alpha0list,alpha1list,grid) {
     plot(c(0,2),c(-1,1),type='n',xlab = "alpha1",ylab = "alpha0")
     n0 = length(alpha0list)
     n1 = length(alpha1list)
     for (a0 in 1:n0) {
          for (a1 in 1:n1){
               if (grid[[a0,a1]] == 1) {points(alpha1list[[a1]],alpha0list[[a0]],pch=19)}
               }
          }
}
# ----------------------------------------------------------------------------          
# find the boundary (inner) of the identified set - not used.
getboundary = function(gridobject,alpha0list,alpha1list) {
     n0 = length(alpha0list)
     n1 = length(alpha1list)
     verticesup = NULL
     verticesdown = NULL
     for (i1 in (1:n1)) {
         if (max(gridobject[,i1])==1) {
              temp1 = range(alpha0list[gridobject[,i1]==1])
              verticesup=rbind(verticesup,c(alpha1list[i1],temp1[[2]]))
              verticesdown=rbind(verticesdown,c(alpha1list[i1],temp1[[1]]))
              }
     }
     vertices = rbind(verticesup,verticesdown)
     
return(vertices)
}
# ----------------------------------------------------------------------------          
# get coordinates of all points in the identified set in prepation for chull
getset = function(gridobject,alpha0list,alpha1list) {
     n0 = length(alpha0list)
     n1 = length(alpha1list)
     coordinates = NULL
     for (i1 in (1:n1)) {
          for (i0 in (1:n0)) {
               if (gridobject[[i0,i1]] == 1) {
                    coordinates = rbind(coordinates,c(alpha1list[[i1]],alpha0list[[i0]]))
                    }
               }
          }
     return(coordinates)
}

