# first version: 26 Oct 2008
# this version: 2 Nov 2009

# INFO: 
# 'radar' searches a boundary of a 2-dim set (possible extension to 3-dim)
# Restrictions:
#	a) set is closed and bounded
#	c) initial searching point has to be in the set

# ALGORITHM:
#	Cartesian coordinates are changed into polar coordinates (psi,r) 
#	where psi is an angle from 0X coordiante and r is a radius.
#	Algorithm searches for points INSIDE and OUTSIDE the set that are eps-distant from each other;
#	search goes along r(psi) and psi is predefined on a grid (psistep determines a size of psi grid)

# USAGE:
# 	The most important function is radar() and a useful one is plotRadar(). 
#	Remaining procedures are auxiliary. Basic usage of radar() is:

#		 radar(initpt,confun)
# 	where:
#	initpt - initial point IN the set
#	confun - a function determining wheather a point lies in/out of the set;
#			 Input to confun is a point! Output is TRUE/FALSE

# --------------------------------------------------------------------------
# draw a point
plotPt <- function(pt,col="black") points(pt[1],pt[2],col=col,lwd=.9,cex=0.5,pch=21,bg=col)
# --------------------------------------------------------------------------
# starting from a point IN the set search for a boundary along direct vector
hitBound <- function(pt,confun,direct=c(1,0),step=0.5,eps=0.1,plt=FALSE,idx=0){
	while( confun(pt) ){ 
		ptin <- pt
		pt <- pt + direct*step
		step <- step*2
		idx <- idx+1
		if(plt) plotPt(pt)
	}
	if(plt) plotPt(pt,col="orange")
return(list(ptin=ptin,ptout=pt,idx=idx))
}
# --------------------------------------------------------------------------
# go OUT of the set along direct vector
goOut <- function(pt,confun,direct=c(0,1),step=0.2,eps=0.1,plt=FALSE,idx=0){
	while( confun(pt) ){ 
		pt <- pt+direct*step
		idx <- idx + 1
		if(plt) plotPt(pt)
	}
return(list(pt=pt,idx=idx))
}
# --------------------------------------------------------------------------
# go INTO the set along direct vector
goIn <- function(pt,confun,direct=c(0,1),step=0.2,eps=0.1,plt=FALSE,idx=0){
	while( !confun(pt) ){ 
		pt <- pt-direct*step
		idx <- idx+1
		if(plt) plotPt(pt)
	}
return(list(pt=pt,idx=idx))
}
# --------------------------------------------------------------------------
# Bisection search algorithm
# give two points A nd B (inside and outside the set) search for 
# A' and B' (outside and inside the set) lying on the interval AB s.t. distance d(A',B') < eps
# otherwords: find eps-closure of a boundary 

bisectSearch <- function(ptin,ptout,confun,eps=0.1,plt=FALSE,idx=0){
	ptdis <- sqrt((ptin[1]-ptout[1])^2+(ptin[2]-ptout[2])^2) #	parlen <- 999
	while(ptdis > eps){
		ptmid <- c((ptout[1]+ptin[1])/2,(ptout[2]+ptin[2])/2)
		idx <- idx+1
		if(plt) plotPt(ptmid)
		if(confun(ptmid)) ptin <- ptmid
		else ptout <- ptmid
		ptdis <- sqrt((ptin[1]-ptout[1])^2+(ptin[2]-ptout[2])^2)
	}
	if(plt){ plotPt(ptin,col="red"); plotPt(ptout,col="blue")}
	
return(list(ptin=ptin,ptout=ptout,idx=idx))
}
# --------------------------------------------------------------------------
# starting from the point within the set:
# 	a) find a boundary in a direction given by  ( sin(psi),cos(psi) )
#	b) traverse along the boundary in polar coordinates (0:2*pi)
radar <- function(initpt,confun,psi=pi/2,eps=.01,step=.3,psistep=0.1*pi,plt=FALSE,ctrl=FALSE,idx=0){
	if(!confun(initpt)) return("Radar Stoped: Initial point outside the set.")
	timeini <- proc.time()
	psivec <- seq(from=(psi+psistep),to=(2*pi+psi),by=psistep)
	ptsvec <- matrix(0, nrow=(length(psivec)-1),ncol=4)
	if(ctrl) cat("\n psi=",round(psi,3)," ; eval ",1," of ",length(psivec)," psi-grid components.") 

	rvec <- c(sin(psi),cos(psi))
	bpts <- hitBound(initpt,confun=confun,direct=rvec,eps=eps,step=step,plt=plt,idx=idx)
	idx <- bpts$idx
	boundpts <- bisectSearch(ptin=bpts$ptin,ptout=bpts$ptout,confun=confun,eps=eps,plt=plt,idx=idx)
	ptin <- boundpts$ptin
	ptout <- boundpts$ptout
	idx <- boundpts$idx

	for(i in 1:(length(psivec)-1)){
		if(ctrl) cat("\n psi=",round(psivec[i],3)," ; eval ",i+1," of ",length(psivec)," psi-grid components. Time elapsed: ",(proc.time()-timeini)[3],",s",sep="") 
		rvec <- c(sin(psivec[i]),cos(psivec[i]))
		disptin <- sqrt( sum( (ptin-initpt)^2) )
		disptout <- sqrt( sum( (ptout-initpt)^2) )
		ptin <- initpt + rvec*disptin
		ptout <- initpt + rvec*disptout
		idx <- idx+2 # +2 because two if'y function below
		if( !confun(ptin) ){
			temp <- goIn(ptin,direct=rvec,confun=confun,eps=eps,plt=plt,idx=idx)
			ptin <- temp$pt
			idx <- temp$idx			
		}
		if( confun(ptout) ){
			temp <- goOut(ptout,direct=rvec,confun=confun,eps=eps,plt=plt,idx=idx)
			ptout <- temp$pt
			idx <- temp$idx
		}
		boundpts <- bisectSearch(ptin=ptin,ptout=ptout,confun=confun,eps=eps,plt=plt,idx=idx)
		ptin <- boundpts$ptin
		ptout <- boundpts$ptout
		idx <- boundpts$idx + 1
		ptsvec[i,] <- c(ptin,ptout)
	}
	time <- (proc.time()-timeini)[3]
	res <- list(ptinvec=ptsvec[,1:2],
				ptoutvec=ptsvec[,3:4],
				psi=psi,
				psistep=psistep,
				psivecln=length(psivec),
				step=step,
				eps=eps,
				idx=idx,
				initpt=initpt,
				time=time
				)
	class(res) <- "radar"
	return(res)
}
# --------------------------------------------------------------------------
print.radar <- function(x,...){
	inipsi <- x$psi/pi
	psistep <- x$psistep/pi
	nIN <- x$psivecln
	nOUT <- x$psivecln
	nTot <- nIN + nOUT
	nEval <- x$idx
	eps <- x$eps
	min <- as.integer(x$time/60)
	sec <- x$time - 60*min
		
	cat("\n Results from Radar Function:")
	cat("\n --------------------------------------")
	cat("\n Initial psi : ",round(inipsi,3),"*pi",sep="")
	cat("\n psistep     : ",round(psistep,3),"*pi",sep="")
	cat("\n\n No of points IN the set      : ",nIN,sep="")
	cat("\n No of points OUTSIDE the set : ",nOUT,sep="")
	cat("\n No of valid boundary points  : ",nTot,sep="")
	cat("\n No of evaluations            : ",nEval,sep="")
	cat("\n Max distance of IN-OUT pair  : ",round(eps,3),sep="")

	if(min!=0) cat("\n\n Search time: ",min,"m ",sec," s",sep="")
	else cat("\n\n Search time: ",sec," s",sep="")
	cat("\n --------------------------------------")
res <- list(inipsi=inipsi,psistep=psistep,nIN=nIN,nOUT=nOUT,nTot=nTot,nEval=nEval,eps=eps,tmin=min,tsec=sec)
invisible(res)
}
# --------------------------------------------------------------------------
print.ProbitSet <- function(x,...){
	cat("\n Ordered Probit Settings:")
	cat("\n --------------------------------------")
	cat("\n alpha : ",x$a)
	cat("\n beta  : ",x$b)
	cat("\n sigma_vv     : ",x$svv)
	cat("\n sigma_uv     : ",x$suv)
	cat("\n\n Tau : ",round(x$tauvec,2))
	cat("\n Tm  : ",round(x$Tm,2))
	cat("\n --------------------------------------")
}
# --------------------------------------------------------------------------
plotRadar <- function(x,
					type="polygon",
					main=NULL,
					colin="whitesmoke",
					colout="black",
					xlim=c(-1,1),
					ylim=c(0,3),
					inner=TRUE,
					outer=TRUE,...){
	
	if(class(x)=="list") nsets <- length(x)
	else nsets <- 1
	lalpha <- 30
	alpha1 <- seq(from=xlim[1],to=xlim[2],len=lalpha)
	alpha2 <- seq(from=ylim[1],to=ylim[2],len=lalpha)
	grd <- expand.grid(x=alpha1,y=alpha2) 

	if(type=="points" && is.null(main)) main <- "Boundary Points"
	if(type=="polygon" && is.null(main)) main <- ""
	plot(grd[,1],grd[,2],col="white",lwd=.5,cex=0.3,xlab=expression(alpha[0]),ylab=expression(alpha[1]),main=main)

	for(i in 1:nsets){
		ifelse( length(colin)==length(x),tcolin <- colin[i], tcolin <- colin)
		ifelse( length(colout)==length(x),tcolout <- colout[i], tcolout <- colout)
		ifelse(nsets != 1, xtemp <- x[[i]], xtemp <- x)
		ptinvec <-  xtemp$ptinvec
		ptoutvec <- xtemp$ptoutvec
#		idx <- seq(from=1,to=100,by=2)
#		ptinvec <- ptinvec[idx,]
#		ptoutvec <- ptoutvec[idx,]
		if(type=="polygon"){
			if(outer) polygon(ptoutvec[,1],ptoutvec[,2], col=tcolout, lty=1, lwd=1,border=tcolout)
			if(inner) polygon(ptinvec[,1],ptinvec[,2], col=tcolin, lty=1, lwd=1,border=tcolin)			
		}
		if(type=="points"){
			plotPt(x$initpt,col="green")
			if(outer) for(i in 1:nrow(ptoutvec)) plotPt(ptoutvec[i,],col=tcolout)
			if(inner) for(i in 1:nrow(ptinvec)) plotPt(ptinvec[i,],col=tcolin)
		}
	}
}

